@@ -95,7 +95,7 @@ cc1 <- capture.output(print(summary(fm3)))
 expect_true(any(grepl("Dispersion model:",cc0)))
 expect_true(any(grepl("Dispersion model:",cc1)))
 
-
+## FIXME: slow ( ~ 49 seconds )
 ## ??? wrong context?
 # not simulated this way, but returns right structure
 test_that("weird variance structure", {
@@ -167,10 +167,25 @@ c3 <- getVCText(vc,comp=c("Variance"))
 expect_equal(c3,structure(list(V3 = c(4.814071, 0.046192, 1.716208), V4 = c(NA, 
 -0.581, NA)), .Names = c("V3", "V4"), class = "data.frame", row.names = c(NA, 
 -3L)),
-tolerance=1e-5)
+tolerance=5e-5)
 
 if (FALSE) {  ## not yet ...
     as.data.frame(vc)
     as.data.frame(vc,order="lower.tri")
 }
 
+Orthodont$units <- factor(seq(nrow(Orthodont)))
+## suppress 'false convergence' warning
+suppressWarnings(fm0 <- glmmTMB(distance ~ age + (1|Subject) + (1|units),
+               dispformula=~0,
+               data = Orthodont))
+
+test_that("VarCorr omits resid when dispformula=~0", {
+    expect_false(attr(VarCorr(fm0)$cond,"useSc"))
+    ## Residual vars not printed
+    expect_false(any(grepl("Residual",capture.output(print(VarCorr(fm0))))))
+})
+
+test_that("vcov(.,full=TRUE) works for zero-disp models", {
+    expect_equal(dim(vcov(fm0,full=TRUE)),c(4,4))
+})
